package jdbernard.timestamper.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 *
 * @author Jonathan Bernard ({@literal jonathan.bernard@gemalto.com})
 */
public class TimelineProperties {

    public static final String LOCAL_TIMELINE_URI = "timeline.uri";
    public static final String REMOTE_TIMELINE_BASE = "remote.timeline.";

    private static final Pattern remoteTimelinePropPattern = 
            Pattern.compile("\\Q" + REMOTE_TIMELINE_BASE + "\\E([^\\s\\.=]+?)[\\.=].*");

    private File propertyFile;
    private Timeline timeline;
    private TimelineSource timelineSource;
    private LinkedList<SyncTarget> syncTargets = new LinkedList<SyncTarget>();

    public TimelineProperties() {
        this.propertyFile = new File("timeline.default.properties");
        Properties config = new Properties();

        File timelineFile = new File("timeline.default.txt");
        URI timelineURI = timelineFile.toURI();

        timeline = new Timeline();
        timelineSource = TimelineSourceFactory.newInstance(timelineURI);
        try { timelineSource.persist(timeline); }
        catch (IOException ioe) {
            // TODO
        }

        config.setProperty(LOCAL_TIMELINE_URI, timelineURI.toString());

        try { config.store(new FileOutputStream(propertyFile), ""); }
        catch (IOException ioe) {
            // TODO
        }
    }

    public TimelineProperties(File propertyFile) throws IOException {
        String strURI;
        URI timelineURI;

        this.propertyFile = propertyFile;
        Properties config = new Properties();
        try {
            config.load(new InputStreamReader(new FileInputStream(propertyFile)));
        } catch (IOException ioe) {
            // TODO
        }

        // load local timeline
        strURI = config.getProperty(LOCAL_TIMELINE_URI, "");
        if ("".equals(strURI)) {
            timelineURI = new File("timeline.default.txt").toURI();
        } else {
            try { timelineURI = new URI(strURI); }
            catch (URISyntaxException urise) {
                throw new IOException("Unable to load the timeline: the timeline "
                        + "URI is invalid.", urise);
            }
        }

        timelineSource = TimelineSourceFactory.newInstance(timelineURI);
        timeline = timelineSource.read();

        // search keys for remote timeline entries
        for (Object keyObj : config.keySet()) {
            if (!(keyObj instanceof String)) continue;

            String key = (String) keyObj;
            String stName;
            String remoteBase;
            SyncTarget st;

            Matcher m = remoteTimelinePropPattern.matcher(key);
            if (!m.matches()) continue;

            stName = m.group(1);
            remoteBase = REMOTE_TIMELINE_BASE + stName;

            strURI = config.getProperty(remoteBase + ".uri", "");
            try { timelineURI = new URI(strURI); }
            catch (URISyntaxException urise) { /* TODO */ }

            // add a new SyncTarget to the list
            st = new SyncTarget(stName, TimelineSourceFactory
                    .newInstance(timelineURI), timeline);
            syncTargets.add(st);

            // check for synch options
            st.enablePull(Boolean.parseBoolean(
                    config.getProperty(remoteBase + ".pull", "true")));
            st.enablePush(Boolean.parseBoolean(
                    config.getProperty(remoteBase + ".push", "true")));
            st.enableSyncOnExit(Boolean.parseBoolean(
                    config.getProperty(remoteBase + ".sync-on-exit", "true")));
            st.setSyncInterval(Long.parseLong(
                    config.getProperty(remoteBase + ".update-interval",
                    "1800000"))); // thirty minutes

        }
    }

    public void save() throws IOException {
        Properties config = new Properties();
        timelineSource.persist(timeline);

        config.setProperty(LOCAL_TIMELINE_URI,
                timelineSource.getURI().toString());

        for (SyncTarget st : syncTargets) {
            String remoteBase = REMOTE_TIMELINE_BASE + st.getName();
            config.setProperty(remoteBase + ".uri",
                    st.getSource().getURI().toString());
            config.setProperty(remoteBase + ".pull",
                    Boolean.toString(st.isPullEnabled()));
            config.setProperty(remoteBase + ".push",
                    Boolean.toString(st.isPushEnabled()));
            config.setProperty(remoteBase + ".sync-on-exit",
                    Boolean.toString(st.isSyncOnExitEnabled()));
            config.setProperty(remoteBase + ".update-interval",
                    Long.toString(st.getSyncInterval()));
        }

        try {
            config.store(new FileOutputStream(propertyFile), "");
        } catch (IOException ioe) {
            // TODO
        }
    }

    public void save(File newFile) throws IOException {
        propertyFile = newFile;
        save();
    }

    public Timeline getTimeline() { return timeline; }

    public void setTimelineSource(TimelineSource newSource) {
        this.timelineSource = newSource;
    }

    public TimelineSource getTimelineSource() { return timelineSource; }

    public Collection<SyncTarget> getSyncTargets() { return syncTargets; }
}
